% Konstantina Palla <kp376@cam.ac.uk>
% University of Cambridge
% Created: August 2012
% Modified: August 2012

function [param, settings] = check_settings(param, settings)

% function that checks that the settings provided by the user are correct.


if size(param.R,1)==0
     err = MException('Initialisation:MtxNotProvided', ...
        'the link matrix R is not provided. Please start again.');
    throw(err)
elseif size(param.R,1) ~=size(param.R,2)
     err = MException('Initialisation:MtxNotProvided', ...
        'the link matrix R is not square. Please choose again.');
    throw(err)
end


if isequal(settings.z_init,'given') 
    if size(param.Zinput,1)~=size(param.R,1)
     err = MException('Initialisation:MtxNotProvided', ...
        'the number of the input matrix Z should be equal to the size of the link matrix R. Please choose again.');
    throw(err)
    end
end

if ~isequal(settings.sampler_z,'metropolis_hastings') && ~isequal(settings.sampler_z,'finite') && ~isequal(settings.sampler_z,'constant')
    err = MException('Initialisation:MtxNotProvided', ...
        'the sampling method for Z is not correct. Please choose again.');
    throw(err)
end

if ~isequal(settings.z_init,'sequential') && ~isequal(settings.z_init,'given') && ~isequal(settings.z_init,'default')
    err = MException('Initialisation:MtxNotProvided', ...
        'the initialisation method for matrix Z is not correct. Please choose again.');
    throw(err)
end


if ~isequal(settings.sampler_c,'omit') && ~isequal(settings.sampler_c,'constant') && ~isequal(settings.sampler_c,'auxiliary_variables')
    err = MException('Initialisation:MtxNotProvided', ...
        'the sampling method for C is not correct. Please choose again.');
    throw(err)
end


if ~isequal(settings.sampler_w,'slice_sampling_fast') && ~isequal(settings.sampler_w,'slice_sampling') && ~isequal(settings.sampler_w,'metropolis_hastings')
    err = MException('Initialisation:MtxNotProvided', ...
        'the sampling method for W is not correct. Please choose again.');
    throw(err)
end



if isequal(settings.sampler_z,'constant')
    % 1) Detect the error.
    if size(param.Zinput,1)==0
        
        % 2) Construct an MException object to represent the error.
        err = MException('Initialisation:MtxNotProvided', ...
            'Zinput matrix is not provided');
        throw(err)
    end
end

if isequal(settings.z_init,'given')
    % 1) Detect the error.
    if size(param.Zinput,1)==0
        
        % 2) Construct an MException object to represent the error.
        err = MException('Initialisation:MtxNotProvided', ...
            'Zinput matrix is not provided');
        throw(err)
    end
    
end


if isequal(settings.sampler_c,'constant')
    % 1) Detect the error.
    if ~isequal(settings.sampler_z,'constant')
        
        % 2) Construct an MException object to represent the error.
        err = MException('Initialisation:WrongComb', ...
            'You cannot choose ''constant'' option for sampling C when sampling Z is not ''constant'' ');
        throw(err)
    end
    
    if size(param.Cinput,1)==0
        
        % 2) Construct an MException object to represent the error.
        err = MException('Initialisation:MtxNotProvided', ...
            'Cinput or Iinput matrix is not provided');
        throw(err)
    end
    
end


end

