% Konstantina Palla <kp376@cam.ac.uk>
% University of Cambridge
% Created: February 2012
% Modified: August 2012

function [param settings person] = initModel(param, settings)

% Give it a random seed or a fixed one
switch settings.random_seed
    case 1
        settings.seed=sum(100*clock);
        rand('state',settings.seed);
        randn('state',settings.seed);
    case 0
        settings.seed=settings.seed;
        
        rand('state',settings.seed);
        randn('state',settings.seed);
        
end

% create test and train mask. Make sure they are SYMMETRIC
[settings.train_mask settings.test_mask]= create_masks(param.R, settings);

%initialize hyperparameters
param.bias_iter=zeros(1,settings.iterations);
% switch settings.sample_bias
%     case 1
        param.bias=param.bias_mu+param.bias_sigma*randn(1,1);
%     case 0
%         param.bias=param.bias;
% end


param.alpha_iter=zeros(1,settings.iterations);
% switch settings.sample_alpha
%     case 1
%         param.alpha= gamrnd(1,1);
%     case 0
        param.alpha=param.alpha;
% end


param.ggamma_iter=zeros(1,settings.iterations);
% switch settings.sample_gamma
%     case 1
%         param.ggamma= exprnd(1);
%     case 0
        param.ggamma=param.ggamma;
% end



test_mask=settings.test_mask;
train_mask=settings.train_mask;




%%%%%%%%%%%%%%%%%%%%%%%%%%% Initialize Markov Chain State %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N=size(param.R,1);

% structure to store the Zs matrices over the iterations
param.Zs_iter = cell(settings.iterations , 1);


% randomly permute the objects
person=randperm(N);
vec=[];



%%%Initialize Z matrix %%%%
switch settings.sampler_z
    case 'constant'
        Z_init = param.Zinput;
        Rin =param.R;
        M_init=  size( Z_init , 2 );
        
    case 'finite'
        switch settings.z_init
            case 'sequential'
                Z_init = ones(1,settings.M_finite);
                M_init=settings.M_finite;
                Rin = param.R(person(1),person(1));
                settings.test_mask=test_mask(vec,vec);
                settings.train_mask=train_mask(vec,vec);
               
            case 'given'
                Z_init = param.Zinput;
                Rin =param.R;
                M_init=  size( Z_init , 2 );
                
            case 'default'
                %Z_init = round(rand(N,settings.M_finite));
                [Z_init M_init] = ibp_generate(N, param.alpha, settings.M_finite);
                
                Rin=param.R;
                
        end
        
    case 'metropolis_hastings'
        switch settings.z_init
            
            
            case 'sequential'
                Z_init = [1];
                vec=person(1);
                Rin = param.R(vec,vec);
                settings.test_mask=test_mask(vec,vec);
                settings.train_mask=train_mask(vec,vec);
               
                M_init=  size( Z_init , 2 );
                
            case 'given'
                Z_init = param.Zinput;
                Rin =param.R;
                M_init=  size( Z_init , 2 );
                
            case 'default'
                %                 Z_init = ones(N,1);
                [Z_init M_init] = ibp_generate(N, param.alpha);
                Rin=param.R;
                
                
        end
        
end
param.Zscur=Z_init;


%structure to store the number of features/columns during the iterations
param.M_iter =  cell(settings.iterations , 1);
assert(M_init==  size( param.Zscur , 2 ));
param.Mcur= M_init;

%structure to store the number of clusters found in each clustering
%over the iterations
%initialize all clusterings with one cluster
param.Ks_iter = cell(settings.iterations,1);




%structure to store the partitions (1 to M) during
%each iteration
param.Cs_iter = cell(settings.iterations, 1);
param.Cscur = cell(1, M_init);


%structure to store the number of people assigned to each clustering
% over the iterations
param.Ns_iter  = cell(settings.iterations, 1);
param.Nscur = sum(Z_init,1);


%structure to store the capacity of each cluster in each clustering
param.NNs_iter = cell(settings.iterations, 1);
param.NNscur = cell(1, M_init);


% structure to store the indices of the persons that belong to each
% clustering
param.Is_iter= cell(settings.iterations, 1);
param.Iscur = cell(1, M_init);


% initialize clusters' assignment
% initialize partitions inside each clustering

switch settings.sampler_c
    case 'constant'
        param.Cscur = param.Cinput;
        for m=1:param.Mcur
            param.Kscur(m) = length(unique(param.Cscur{m}));
            param.Iscur{m} = find(param.Zscur(:,m));
            un = unique(param.Cscur{m});
            for r=1:param.Kscur(m)
                param.NNscur{m}(r)=length(find(param.Cscur{m}==un(r)));
            end
        end
        
        
    case {'auxiliary_variables', 'omit'}
        param.Kscur=ones(1, M_init);
        for m=1:M_init
            
%             param.Cscur{m} = ceil(rand(1, param.Nscur(m))* param.Kscur(m));
                        param.Cscur{m} = ones(1, param.Nscur(m));
            %              param.Cscur{m}= crpgen(param.Nscur(m), param.ggamma);
            param.Kscur(m)=length(unique(param.Cscur{m}));
            
            param.Iscur{m} = find(Z_init(:,m));
            un = unique( param.Cscur{m});
            %sanity check
            if length(un)~=  param.Kscur(m)
                disp('problem in initializing the clusters')
            end
            
            
            for r=1:param.Kscur(m)
                
                param.NNscur{m}(r)=length(find( param.Cscur{m}==un(r)));
            end
        end
end

% There will be M weight matrices (W), one for each possible clustering
% W - real valued KxK matrix, w_ij ~ N(0, sigma_w^2)
param.Ws_iter = cell(settings.iterations, 1);
param.Wscur = cell(1, M_init);
for m=1:M_init
    param.Wscur{m} =  param.mu_w+ param.sigma_w.*randn( param.Kscur(m), param.Kscur(m));
end
% for m=1:M_init
%     t=rand;
%     if r<0.5
%         param.Wscur{m} =  param.mu_w1+ param.sigma_w1.*randn( param.Kscur(m), param.Kscur(m));
%     else
%         param.Wscur{m} =  param.mu_w2+ param.sigma_w2.*randn( param.Kscur(m), param.Kscur(m));
%     end
% end



[param.lk  param.T]=compute_likelihood(Rin,  param.Zscur,  param.Cscur,  param.Iscur,  param.Wscur, param, settings);
param.Rin=Rin;

% structure to store the energy
param.E_iter = cell( settings.iterations,1);

settings.test_mask=test_mask;
settings.train_mask=train_mask;

end
