% Konstantina Palla <kp376@cam.ac.uk>
% University of Cambridge
% Created: February 2012
% Modified: August 2012

function [param settings]= load_settings(param, settings)

% Give it a random seed or a fixed one
settings.random_seed=0;
settings.seed=1;

%% Hyperparameters

param.mu_w = 2;
param.sigma_w =4;

param.bias_mu=-2;
param.bias_sigma=1;

%hyperparameters of the prior on alpha, the IBP concentration parameter
param.alpha_shape=1;
param.alpha_scale=1;

%Default values for the parameters
% Gamma parameter for CRP(\gamma)
param.ggamma = 1;

% Alpha parameter for IBP(\alpha)
param.alpha=1; 

% Bias parameter 
param.bias=-2; 


%% Experimental Settings
% number of iterations
settings.iterations=20;
% number of sampling iterations during each step of the seuential
% initialisation
settings.in_itr = 15;
% the number of iterations that will be used for the computation of the
% errror. This is the number of samples taken into account from the end of
% the sampling process. (the ones proceeded will be considered as burn in
% period)
settings.evaluation_window=0.3*settings.iterations;
% the number of auxiliary variables used in Algorithm 8 (Neal's) during the
% C sampling procedure.
settings.m_aux =5;
% define whether the test and train mask is given or is randomly created
settings.masks_given=0;
% percentage of the data links kept as test data for the randomly created masks.
settings.hold_out_per=0.2;


% number of maximum number of features Z matrix should have in the finite
% case
settings.M_finite =8;

% choose whether or not to sample the \gamma (CRP concentration), \alpha
% (IBP concentration) or bias parameters
% set 1 (ON) or 0 (OFF) correspondingly
settings.sample_alpha = 1;
settings.sample_gamma = 1;
settings.sample_bias= 1;

% set the sampling methods for the parameters Z, C and W
% choose sampler for z
% options: 'metropolis_hastings', 'finite', 'constant'
settings.sampler_z = 'metropolis_hastings';

% choose initialisation method for z
% options:
%        'sequential': initialises z sequentially,
%        'given': initializes z as it given in param.Z,
%        'default': Z is randomly initialized, random binary matrix 
settings.z_init ='default';

% choose sampling method for C:
%   'constant': the C is not being sampled at all. It remains as it is given in Cinput. This SHOULD be applied
%   along with the 'constant' option for sampling Z.
%   'omit': the C is not being sampled independently from Z. When the
%   option for sampling Z is any apart from 'constant' and the sampling
%   option for C is
%   any apart from 'omit', the C is being sampled both inside the Z and the C sampling procedure.
%   'auxiliary_variables': auxiliary variables method (Radford's Algorithm
%   8)
settings.sampler_c = 'auxiliary_variables';

% choose sampling method for w:
% options: 'slice_sampling_fast'
settings.sampler_w = 'slice_sampling_fast';

% sampler for alpha:
% 'slice' or 'exact'. Exact may be chosen because the prior is a gamma
% distribution
settings.sampler_alpha='exact';

% plot Z, C and W during each iteration. Choose 0 (OFF) or 1 (ON)
settings.plot_matrices=1;

end

