% Konstantina Palla <kp376@cam.ac.uk>
% University of Cambridge
% Created: February 2012
% Modified: August 2012

addpath('utils/')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  True matrices %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Create R matrix (NxN)
% This is ALWAYS needed. It is the observed data. Should be a NxN matrix.
% You may as well load it.
N=30;
Rtest1 = zeros(N,N);
Rtest1(1:10,1:10)=1;
Rtest1(11:20,11:20)=1;
Rtest1(21:30,21:30)=1;
Rtest1(6:15,16:25)=1;
Rtest1(16:25,6:15)=1;
param.R=Rtest1;

%% Create Z matrix 
% This might be either the true Z or the one you want the sampler to start
% with. For the latter case, make sure to set
% settings.sampler_z='given'. Otherwise set param.Zinput=[];

Z= ones(1,N)';
temp=zeros(1,N)';
temp(6:25)=1;
Z=[temp Z ];
param.Zinput=Z;

%% Create C matrix 
% This will be the true C if known and will be used in the case you want to 
% set setting.sampler_c='constant'. In that case. C won't be sampled at all
% but will be kept fixed at its true value. If not known, just set 
% param.Cinput={};
C{1}=ceil((1:20)/10);
C{2}=ceil((1:N)/10);
param.Cinput=C;




%% Load the settings
[param settings] = load_settings(param);


%%  Set the parameters %%
% Initialise the parameters (Your choice will be overwritten if you choose to 
% sample them. Look at 'load_settings.m file'). 


% Gamma parameter for CRP(\gamma)
param.ggamma = exprnd(1);
% param.ggamma = 1;

% Alpha parameter for IBP(\alpha)
param.alpha = gamrnd(param.alpha_shape,param.alpha_scale);
% param.alpha=1; 

% Bias parameter 
param.bias=param.bias_mu+param.bias_sigma*randn(1,1);
% param.bias=-2; 



%% Run the INFERENCE
[param settings] = inference(param, settings);


%% Error estimation
[ err ] = compute_err( param, settings );


%% Compute the Area Under the Curve Estimate
auc = compute_AUC(param, settings, err);

    

%% Save results
 filename=strcat('ila_result_', int2str(settings.seed), '.mat');
 save(filename, 'param', 'settings', 'err', 'auc');

 
