% Konstantina Palla <kp376@cam.ac.uk>
% University of Cambridge
% Created: February 2012
% Modified: August 2012

function [param] = sample_z_single_iter(R, param, settings)

% function that implements one iteration of sampling IBP matrix Z
% infinite case

alpha = param.alpha;
mu_w = param.mu_w ;
sigma_w = param.sigma_w;
ggamma = param.ggamma;

Zs=param.Zscur;
Cs=param.Cscur;
Is=param.Iscur;
Ks=param.Kscur;
Ns=param.Nscur;
NNs=param.NNscur;
Ws=param.Wscur;

% 1. For non-zero columns, we must first be able to sample
%    z_im|Z¬im, Cs, W, R  ~ p(R| Z¬im, zim, Cs, Ws)p(z_im |Z¬im)
% 2. For each row we must also be able to sample the number of new features unique
%    to the ith row

N=size(Zs, 1);
single_col_set = zeros( 1 , 1000 );
perm=randperm( N );
for ind=1:N
    i=perm(ind);
    %number of current features in Z
    M = size(Zs, 2);
    num_single_col = 0;
    
    % FOR the NON-zero Columns
    % we only need to consider z_im= {0,1}; so we evaluate the righthand
    % side for z_im = 0 and z_im = 1, normalize, and sample z_im from the resulting
    % Bernoulli distribution.
    
    permM=randperm( M );
    
    for indm=1:M
        m = permM(indm);
        
        % The number of data points for which this feature is active
        % excluding the current one.
        s = sum(Zs(:,m)) - Zs(i,m);
        if s == 0
            if Zs(i,m) == 1
                num_single_col = num_single_col + 1;
                single_col_set( num_single_col ) = m;
            end
            
        else
            
            % p(z_im|Z¬im) conditional prior
            log_conditional_prior = log([(N-s)/N   s/N]);
            
            
            %%%%%%%%%%%%%%%%%%%%
            % compute the prob of turning the feature on, that is z_im=1.
            % need to integrate over the possible c_im's values.            
            [Zs1, Cs1, Is1, Ks1, Ns1, NNs1, Ws1, lk1, logl1, T1]=turn_feature_on(R, i, m, Zs, Cs, Is, Ks, Ns, NNs, Ws, param, settings);
           
            % compute the likelihood for the case of turning off                                      
            [Zs0, Cs0, Is0, Ks0, Ns0, NNs0, Ws0, logl0, T0]=turn_feature_off(R, i, m, Zs, Cs, Is, Ks, Ns, NNs, Ws, param, settings);

            lk = [logl0 logl1];
            pr = log_conditional_prior+lk;
            pr = exp(pr-max(pr));
            pr=pr/(sum(pr));
            
            % sample z_im for the non-zero columns
            uu=rand;
            t =sum(uu>cumsum(pr));
            Zs(i, m) = t;
            
            
            
            if (Zs(i,m)==1)
                Zs = Zs1;
                Cs=Cs1;
                Is= Is1;
                Ks=Ks1;
                Ns=Ns1;
                NNs=NNs1;
                Ws=Ws1;
                param.T=T1;
%                 param.lk=lk1;
                
            else
                Zs = Zs0;
                Cs=Cs0;
                Is=Is0;
                Ks=Ks0;
                Ns=Ns0;
                NNs=NNs0;
                Ws=Ws0;
                param.T=T0;
%                 param.lk=logl0;
                
            end
            
        end
        
    end
    
    %     for ch=1:size(Zs,2)
    %         check_zero_col=sum(Zs,1);
    %        if  find(check_zero_col==0)
    %            keyboard
    %
    %        end
    %     end
    
    % For new Columns/Features ********* Metropolis-Hastings *************
    % Propose a number k_new for unique features from the prior
    m_new = poissrnd( alpha / N );
    
      
    if (m_new || num_single_col) > 0
        M = size(Zs,2);
        % augment Z matrix
        Zs_new =Zs;
        Zs_new( i , (end+1):(end+m_new) ) = 1;
        
        % turning on m_new features/columns for object i
        % creates m_new clusterings with one object in each
        % update the relevant structures
        Ns_new=Ns;
        Ns_new(M+1:M+m_new) = 1;
        
        
        Is_new = cell(1, size(Zs_new,2));
        for m=1:size(Zs_new,2)
            Is_new{m} = find(Zs_new(:,m));
        end
        
        Ks_new= Ks;
        Ks_new((end+1):(end+m_new)) =1;
        
        
        Cs_new = Cs;
        
        NNs_new =NNs;
        for m=1:m_new
            Cs_new{end+1} = 1;
            NNs_new{end+1}=1;
        end
        
        Ws_new=Ws;
        for m=1:m_new
         
             Ws_new{M+m}= mu_w+sigma_w * randn(1,1);
        end
%         for m=1:m_new
%              if rand<0.5
%                  Ws_new{M+m} =param.mu_w1+param.sigma_w1*randn(1,1);
%             
%             else
%                 Ws_new{M+m} =param.mu_w2+param.sigma_w2*randn(1,1);
%             
%             end
%         end
%         


        zs_r=Zs_new(:,single_col_set( 1:num_single_col ));
        Zs_new(:,single_col_set( 1:num_single_col ))=[];
        
        
        Ns_new(single_col_set( 1:num_single_col ))=[];
        
        is_r = Is_new(single_col_set( 1:num_single_col ));
        Is_new(single_col_set( 1:num_single_col ))=[];
        Ks_new(single_col_set( 1:num_single_col ))=[];
        
        cs_r =Cs_new(single_col_set( 1:num_single_col ));
        Cs_new(single_col_set( 1:num_single_col ))=[];
        
        NNs_new(single_col_set( 1:num_single_col ))=[];
        
        ws_r=Ws_new(single_col_set( 1:num_single_col ));
        Ws_new(single_col_set( 1:num_single_col ))=[];
        M = M -num_single_col;
     
        [lk_old To]=compute_likelihood_new(i, [], cs_r, is_r, ws_r, R, [], Cs_new(M+1:M+m_new), Is_new(M+1:M+m_new), Ws_new(M+1:M+m_new), param, settings); %check
        
        [lk_new T_new]=compute_likelihood_new(i, zs_r, cs_r, is_r, ws_r, R, Zs_new(:,M+1:M+m_new), Cs_new(M+1:M+m_new), Is_new(M+1:M+m_new), Ws_new(M+1:M+m_new), param, settings);
        
%        [li Ti]= compute_likelihood(R, Zs, Cs, Is, Ws, param, settings);
%        if abs(sum(sum(param.T-Ti)))>10^(-10)
%            sum(sum(param.T-Ti))
%            param.T
%            Ti
% %            pause
%        end
%        [lii Tii]= compute_likelihood(R, Zs_new, Cs_new, Is_new, Ws_new, param, settings)
% pause

        
        r = lk_new-lk_old;
        r =exp(r);
        
        a= min(1,r);
        if (rand<a)
            Zs = Zs_new;
            Cs=Cs_new;
            Is=Is_new;
            Ks=Ks_new;
            Ns=Ns_new;
            NNs=NNs_new;
            Ws=Ws_new;
            param.T=T_new;
%             param.lk=lk_new;
        end
    end
    
end

param.Zscur=Zs;
param.Cscur=Cs;
param.Iscur=Is;
param.Kscur=Ks;
param.Nscur=Ns;
param.NNscur=NNs;
param.Wscur=Ws;

end
