% Konstantina Palla <kp376@cam.ac.uk>
% University of Cambridge
% Created: February 2012
% Modified: August 2012

function [ param] = sample_z_single_iter_finite(R, param, settings)

alpha = param.alpha;

Zs=param.Zscur;
Cs=param.Cscur;
Is=param.Iscur;
Ks=param.Kscur;
Ns=param.Nscur;
NNs=param.NNscur;
Ws=param.Wscur;

% 1. For non-zero columns, we must first be able to sample
%    z_im|Z¬im, Cs, W, R  ~ p(R| Z¬im, zim, Cs, Ws)p(z_im |Z¬im)
% 2. For each row we must also be able to sample the number of new features unique
%    to the ith row

% disp('*********** z sample')

N=size(Zs, 1);
perm=randperm( N );
for ind=1:N
    i=perm(ind);
    %number of current features in Z
    M = size(Zs, 2);
    
    
    % FOR the NON-zero Columns
    % we only need to consider z_im= {0,1}; so we evaluate the righthand
    % side for z_im = 0 and z_im = 1, normalize, and sample z_im from the resulting
    % Bernoulli distribution.
    
    %
    
    
    
    permM=randperm( M );
    
    for indm=1:M
        m = permM(indm);
        
        % The number of data points for which this feature is active
        % excluding the current one.
        s = sum(Zs(:,m)) - Zs(i,m);
      
            
            % p(z_im|Z¬im) conditional prior
            log_conditional_prior = log([(N-s)/(N+alpha/M)   (s+alpha/M)/(N+alpha/M)]);
            
            
            %%%%%%%%%%%%%%%%%%%%
            % compute the prob of turning the feature on, that is z_im=1.
            % need to integrate over the possible c_im's values.
            [Zs1, Cs1, Is1, Ks1, Ns1, NNs1, Ws1, lk1, logl1, T1]=turn_feature_on(R, i, m, Zs, Cs, Is, Ks, Ns, NNs, Ws, param, settings);
            
            % compute the likelihood for the case of turning off
            [Zs0, Cs0, Is0, Ks0, Ns0, NNs0, Ws0, logl0, T0]=turn_feature_off(R, i, m, Zs, Cs, Is, Ks, Ns, NNs, Ws, param, settings);
            
            lk = [logl0 logl1];
            pr = log_conditional_prior+lk;
            pr = exp(pr-max(pr));
            pr=pr/(sum(pr));
            
            % sample z_im for the non-zero columns
            uu=rand;
            t =sum(uu>cumsum(pr));
            Zs(i, m) = t;
            
            
            
            if (Zs(i,m)==1)
                Zs = Zs1;
                Cs=Cs1;
                Is= Is1;
                Ks=Ks1;
                Ns=Ns1;
                NNs=NNs1;
                Ws=Ws1;
                param.T=T1;
                %                 param.lk=lk1;
                
            else
                Zs = Zs0;
                Cs=Cs0;
                Is=Is0;
                Ks=Ks0;
                Ns=Ns0;
                NNs=NNs0;
                Ws=Ws0;
                param.T=T0;
                %                 param.lk=logl0;
                
            end
      
        
    end
    
    
    
    
end

param.Zscur=Zs;
param.Cscur=Cs;
param.Iscur=Is;
param.Kscur=Ks;
param.Nscur=Ns;
param.NNscur=NNs;
param.Wscur=Ws;

end
