% Konstantina Palla <kp376@cam.ac.uk>
% University of Cambridge
% Created: February 2012
% Modified: August 2012


function [ param, settings ] = ...
    sampler_single_iter(R, param, settings )

% function that implements one iteration of the MCMC sampler

% Given Cs, Ws, alpha, gamma and bias, sample posterior Zs
switch settings.sampler_z
    case 'metropolis_hastings'
        [param] = ...
            sample_z_single_iter(R, param, settings);
        
        while size(param.Zscur,2)==0
            [param] = ...
                sample_z_single_iter(R, param, settings);
        end
        
    case 'finite'
        [ param] = sample_z_single_iter_finite( R,  param, settings);
        %     case 'constant'
        %         ;

end

% Sample alpha
if settings.sample_alpha ==1
    switch settings.sampler_alpha
        case 'slice'
            param = slice_sampler_hyper_IBP(param);
        case 'exact'
            param = exact_sampler_hyper_IBP(param);
    end
end

% Given Zs, Ws, alpha, gamma and bias, sample posterior Cs
switch settings.sampler_c
    case  'auxiliary_variables'
        [ param] = ...
            sample_c_single_iter_aux(R, param, settings);
        %     case 'constant'
        %         ;
        %     case 'omit'
        %         ;
   
end

% Given Zs, Cs, alpha, gamma and bias, sample posterior Ws
switch settings.sampler_w

    case  'slice_sampling_fast'
        [param] = ...
            slice_sampling_w_fast(R, param, settings);
end

% Sample gamma
if settings.sample_gamma ==1
    %sample ggamma for the CRP (clusters)
    param = slice_sampler_hyper_CRP(param);
end

% Sample bias
if settings.sample_bias
    param = sample_bias(R, param, settings);
end

end

