% Konstantina Palla <kp376@cam.ac.uk>
% University of Cambridge
% Created: February 2012
% Modified: August 2012


function [ param ] = slice_sampler_hyper_IBP( param)
%
% Consider a vague prior Gamma(1, 1) on alpha
% posterior alpha ∼ Gamma(1 + K, 1 + H_N), where K is the number of
% the active features
%prior_a = Gamma(1,1)

alpha=param.alpha;
Zs=param.Zscur;

% remove parts of z with no usages
% this is necessary for the finite case of sampling Z, since then we 
% might have all-zero columns
zeroInd = find( sum(Zs,1) == 0 );
Zs( :, zeroInd ) = [];

%% slice sampler method
%p(alpha|Z,R) prop p(R|Z)p(Z|alpha)p(alpha)
%p(R|Z) independent of alpha --> constant when alpha changes
%---> p(alpha|Z,R) prop p(Z|alpha)p(alpha)



% prior on Z: p(Z|alpha)

[N K] = size(Zs);
hst = zeros(1,K);
for k=1:K
    hst(k)=bin2dec2(Zs(:,k));
end

K_plus = histc(hst,unique(hst));
dd=0;
for t=1:length(K_plus)
  dd= dd + gammaln(K_plus(t)+1);
end
K_plus=sum(K_plus);
harm = sum(ones(1, N)./(1:N));
fac = 0;

m=sum(Zs,1);
for t=1:K_plus
    fac= fac+ gammaln(N-m(t)+1)+gammaln(m(t)) - gammaln(N+1);
  
end
lpz =@(alpha) (K_plus)*log(alpha)-(alpha*harm)+fac-dd;


% 
% 
% % this gives the same result as the one above (the one commented out)
%  lpz =@(alpha) lpdfIBP( Zs, alpha );

h =@(x) lpz(exp(x))+x-exp(x);
N = 1;
t= slicesample(log(alpha), N,'logpdf', h,'thin', 1, 'burnin', 0);
param.alpha=exp(t);


 

end




