% Konstantina Palla <kp376@cam.ac.uk>
% University of Cambridge
% Created: February 2012
% Modified: August 2012

function [param] = slice_sampling_w_fast(R, param, settings)

 Zs=param.Zscur;
 Cs=param.Cscur;
 Is=param.Iscur;
 Ks=param.Kscur;
 Ns=param.Nscur;
 NNs=param.NNscur;
 Ws=param.Wscur;
 
M = size(Zs, 2);
sigma_w = param.sigma_w;
mu_w = param.mu_w ;
% sigma_w1 = param.sigma_w1;
% mu_w1 = param.mu_w1 ;
% sigma_w2 = param.sigma_w2;
% mu_w2 = param.mu_w2 ;

N = size(R, 1);
% disp('----------------sample w: slice sampling')

for m=1:M
    for c=1:Ks(m)
        for l=1:Ks(m)
            %for each clustering, sample the weight matrix
            % p(w_cl|W_¬cl, C, Z, R) prop p(R|w_cl, W_¬cl, C, Z)*p(w_cl|W_¬cl)
            % p(w_cl|W_¬cl) = p(w_cl) = N(mu_w, sigma_w^2) (prior)
            w_old = Ws{m}(c,l);
            
            log_prior = @(w_cl) -0.5*log(2*pi*sigma_w^2)  -((w_cl-mu_w)^2)/(2*sigma_w^2);
%                log_prior= @(w_cl) -log(2) -0.5*log(2*pi) + log((1/param.sigma_w1)*exp(-((w_cl-param.mu_w1)^2)/(2*(param.sigma_w1^2))) ...
%                    +(1/param.sigma_w2)*exp(-((w_cl-param.mu_w2)^2)/(2*(param.sigma_w2^2))  ));         
%             
            cindex = logical(zeros(1,N));
            cindex(Is{m})=(Cs{m}==c);
            lindex = logical(zeros(1,N));
            lindex(Is{m})=(Cs{m}==l);
            
            Rprime=R(cindex,lindex);
            TprimeBias=param.T(cindex,lindex)-w_old+param.bias;
            maskPrime=settings.train_mask(cindex,lindex);
            
            %likelihood term
            lh = @(w_cl) lkhood(w_cl, Rprime, TprimeBias, maskPrime);
            
            f = @(w_cl) lh(w_cl)+log_prior(w_cl);
            
            N = 1;
            
            Ws{m}(c,l) = slicesample(w_old,N,'logpdf',f,'thin',1,'burnin',0,'width',10);
      
            param.T(cindex,lindex)=param.T(cindex,lindex)-w_old+Ws{m}(c,l);
      
        end
    end
end

    function l=lkhood(w_cl, r, tb, m)
         temp=tb+w_cl;

        a=m.*(r.*temp-log(1+exp(temp)));
        l=sum(sum(a));
        if (isnan(l) | isinf(l))
            keyboard
        end
        
    end
% update stored likelihood
% Tb=param.T+param.bias;
% l=-log(1+exp(Tb));
% ra_train=settings.train_mask.*(l+R.*Tb);
% param.lk = sum(sum(ra_train));




 param.Zscur=Zs;
 param.Cscur=Cs;
 param.Iscur=Is;
 param.Kscur=Ks;
 param.Nscur=Ns;
 param.NNscur=NNs;
 param.Wscur=Ws;

end