function logP = lpdfIBP( z, alpha , beta )
% function logP = lpdfIBP( z, alpha , beta )
% Computes the log prior for an IBP variable.
if nargin == 2
    beta = 1;
end

% remove parts of z with no usages
zeroInd = find( sum(z,1) == 0 );
z( :, zeroInd ) = [];

% compute the likelihood
[N K] = size(z);
powerSet = 2.^(1:N);
KhList = sum(z.*repmat(powerSet',1,K),1);
KhSet = unique( KhList );
KhCount = zeros(1,length(KhSet));
for i = 1:length( KhSet )
    KhCount(i) = length( find( KhList == KhSet(i) ));
end

colCount = sum( z,1 );

logP = K * log( alpha * beta ) ...
    - sum( gammaln( KhCount + 1 ) ) ...
    - alpha * sum( beta ./ ( 1:N - 1 + beta ) ) ...
    + sum( gammaln( colCount ) ) ...
    + sum( gammaln( N - colCount + beta ) ) ...
    - K * gammaln( N + beta );
    